#ifndef __THE_MAZE_OF_GALIOUS_2_ACTIVE_OBJECT
#define __THE_MAZE_OF_GALIOUS_2_ACTIVE_OBJECT


// ACTIVE CLASSES: 

#define CONTROL_LEFT	0
#define CONTROL_RIGHT	1
#define CONTROL_UP		2
#define CONTROL_DOWN	3
#define CONTROL_FIRE1	4
#define CONTROL_FIRE2	5
#define CONTROL_FIRE3	6
#define CONTROL_FIRE4	7



class ActiveOBJECT {
	friend class ActiveNetworkMOG2;
	friend class ActiveMOG2Application;
	friend class ActiveMOG2;
	friend class ActiveMAP;
	friend class ActiveROOM;
	friend class ActiveINTERFACE;
	friend class ActionSCRIPT;
	friend class ConditionSCRIPT;
	friend class ValueSCRIPT;

	friend class GameWindow;
	friend class GameDebugWindow;
	friend class NetworkGameWindow;

	friend bool drawing_order(ActiveOBJECT *o1,ActiveOBJECT *o2);

public:
	ActiveOBJECT(PLACED_OBJECT &o,class ActiveROOM *r,class ActiveINTERFACE *interf);
	ActiveOBJECT(int x,int y,int pID,OBJECT *o,ActiveROOM *r,class ActiveINTERFACE *interf);
	~ActiveOBJECT(void);

	bool save(FILE *fp,MOG2 *game);
	bool load(FILE *fp,MOG2 *game);

	void precycle(void);
	bool cycle(class KEYBOARDSTATE **LVK,int n_LVKs);
	void draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp);
	void draw_collision_mask(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp,int constitution);

	void change_tile(int newtile,int part);

	bool passive(void);
	bool glued(void);
	void recompute_glued_position(void);
	float absolute_scale();
	float absolute_angle();

	bool cmp(ActiveOBJECT *obj);
	bool planep(int i);	// Tests whether the object in present in the plane 'i'
	int get_object_plane(void);

	void absolute_hotspot_coordinates(int *x,int *y);
	void hotspot_coordinates(int *x,int *y);

	int get_dx(void);
	int get_dy(void);
	int get_x(void) {return x;};
	int get_y(void) {return y;};

	ActiveMOG2 *get_game(void);
	ActiveMAP *get_map(void);

	void get_bbox(int *x1,int *y1,int *x2,int *y2);
	void get_absbbox(int *ax1,int *ay1,int *ax2,int *ay2);

	void get_family_bbox(int *x1,int *y1,int *x2,int *y2);
	void get_family_absbbox(int *ax1,int *ay1,int *ax2,int *ay2);
	void get_family_multiroom_bbox(int *x1,int *y1,int *x2,int *y2);
	void get_family_multiroom_absbbox(int *x1,int *y1,int *x2,int *y2);

	void copy_state(ActiveOBJECT *o);
	void is_instance(ActiveOBJECT *o);

	void precision_move(int x_move,int y_move);

	PLACED_OBJECT *search_ladder(int xoffs,int yoffs);

	bool collision(ActiveOBJECT *o,bool hierarchical);
	bool collision(int x,int y,ActiveOBJECT *o,bool hierarchical);
	bool collision(ActiveOBJECT *o,int constitution,int o_constitution,bool hierarchical);
	bool collision(int x,int y,ActiveOBJECT *o,int constitution,int o_constitution,bool hierarchical);
	bool low_level_collision(int ax,int ay,int ax2,int ay2,class MOG2CollisionData *o_cdata,int constitution,bool hierarchical);

	ActiveOBJECT *ActiveOBJECT::collision2(int x,int y,ActiveOBJECT *o,int constitution,int o_constitution,ConditionSCRIPT *s,ActiveOBJECT **foreign_object,bool hierarchical);
	bool low_level_collision2(int ax,int ay,int ax2,int ay2,MOG2CollisionData *o_cdata,int constitution,bool hierarchical);

	bool multiroom(void);

	int get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval);
	void set_variable_value(char *vname,int value);
	void set_variable_value(char *vname,char *value);
	void set_variable_value(char *vname,ActiveOBJECT *value);

	bool offspringp(ActiveOBJECT *o);

	int count_objects(char *name,ConditionSCRIPT *cond,ActiveOBJECT *script_o,ActiveROOM *script_r,ActiveMAP *script_m,ActiveMOG2 *script_g,ActiveINTERFACE *script_i,ActiveMOG2Application *script_a,int script_location);

	void delete_children_in_room(ActiveROOM *r);

private:

	/* General objects colision: */ 
	void standard_collision(void);

	/* Character functions: */ 
	void character_initialization(void);
	bool character_cycle(int *desired_tiles,KEYBOARDSTATE **LVK,int n_LVKs);
	void character_postcycle(void);
	void character_collision(void);

	void find_without_collision_place(int xtolerance,int ytolerance);

	/* Text functions: */ 
	void text_initialization(void);
	void text_initialization(OBJECT *o);
	void text_initialization(ActiveOBJECT *o);
	void text_draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp);

	/* Game window functions: */ 
	void gw_draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp);

	/* Menu functions: */ 
	bool menu_cycle(KEYBOARDSTATE **LVK,int n_LVKs);
	void menu_draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp);

	/* General object functions: */ 
	bool has_to_be_pushed(ActiveOBJECT *o);
	void set_push_object(ActiveOBJECT *o);
	void update_instances(void);
	void reinit_created_objects(int xd,int yd);
	int  highest_priority_special(void);

	void delete_terminated_children(void);


	/* Member variables: */ 
	List<ActiveOBJECT> who_pushed_me; /* List that contains the objects that has actually pushed	*/ 
									  /* This object in this frame.									*/ 
	List<ActiveOBJECT> other_instances; /* List of instances of the same object in other rooms */ 
	List<ActiveOBJECT> objects_created;	/* List of objects created on this frame.	*/ 
										/* If during the collision process, the object has to be moved,	*/ 
										/* all the objects in this list have to be moved too.			*/ 

	/* Object hierarchy: */ 
	ActiveOBJECT *parent;
	List<ActiveOBJECT> independent_children,glued_children;
	int glued_part,glued_link;
	int glued_xoffs,glued_yoffs;
	int relative_precedence;

	/* General object variables: */ 
	int ID;
	int LVK_ID;						/* Which of all the LVKs to take input from.	*/ 
									/* LVK=-1 means that the object can take input	*/ 
									/*        from any of the LVKs					*/ 

	long int old_x,old_y;
	long int x,y;
	int horizontal_circularity_count;
	int vertical_circularity_count;
	int precision_x,precision_y;
	ActiveROOM *room;
	ActiveINTERFACE *interf;
	bool check_collision;
	bool terminate;
	bool state_change,executed_state_change;
	List<VARIABLE> object_variables;
	ActiveOBJECT **state_object_foreign;	/* Object detected by the COLLISIOIN_WITH_OBJECT command */ 
	int damage;
	int object_plane;

	int state,old_state;
	int state_timmer;
	int tile[MAX_PARTS],old_tile[MAX_PARTS];
	int p_offset_p[MAX_PARTS],p_offset_o;

	float old_scale,old_angle;
	int scale;						/* The scaling factor is scale/100.0 */ 
	int angle;						/* In degrees (i.e. 360 is a complete revolution) */ 

	/* character variables: */ 
	int energy;
	int left_key,right_key,up_key,down_key,fire1_key,fire2_key,fire3_key,fire4_key;
	int jumping_inertia;
	int jumping_ctnt,gravity_ctnt;
	int walking_speed,ladder_up_speed,ladder_down_speed;
	ActionSCRIPT *invincible_script;
	bool stopped_jumping;
	PLACED_OBJECT *in_ladder;
	bool looking_right_in_ladder;	/* Keeps track of the direction to which the character is looking	*/ 
									/* while going up or down a ladder.									*/ 
	bool continuous_state_change;

	/* Text variables: */ 
	char *text_text;
	char *text_font;
	int text_size;
	int text_spacing;	/* This is only valid for BITMAP fonts, and specifies the space between characters */ 
	int text_r,text_g,text_b,text_a;
	int text_frame;	/* 0 - No frame		*/ 
					/* 1 - Square frame */ 
					/* 2 - Balloon		*/ 
	int text_frame_width;
	int text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1;
	int text_frame_r2,text_frame_g2,text_frame_b2,text_frame_a2;
	int text_frame_radius;
	int text_frame_point_x,text_frame_point_y;
//	ActionSCRIPT *text_script;
	int text_h_align;	/* 0 - Left
						   1 - Center
						   2 - Right */ 
	int text_v_align;	/* 0 - Top
						   1 - Center
						   2 - Bottom */ 
	int text_max_w;

	/* game window variables: */ 
	int gw_dx,gw_dy;
	int gw_scale;
	int gw_min_hs_x,gw_min_hs_y;
	int gw_max_hs_x,gw_max_hs_y;
	List<ActiveOBJECT> gw_focusing;
	int gw_last_viewx,gw_last_viewy;

	/* menu variables: */ 
	char *menu_title;
	int menu_active;
	int menu_timmer;
	int menu_selected;
	int menu_blink_timmer;
	List<char> menu_items;

	/* Drawing MODE: */ 
	int dm_drawing_mode;	/* 0 - normal   */ 
							/* 1 - shaded   */ 
						    /* 2 - bicolor	*/ 
							/* 3 - reflection	*/ 
							/* 4 - refraction	*/ 
	int dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a;
	int dm_color2_r,dm_color2_g,dm_color2_b,dm_color2_a;
	int dm_factor;
	int dm_blink_f0,dm_blink_f1,dm_blink_speed;

	OBJECT *object;

	AnimSCRIPT *state_anim[MAX_PARTS][CPOS_NSTATES];

	AnimSCRIPT **g_state_anim[MAX_PARTS];
	class ActionSCRIPT **g_state_action; 
	bool g_state_finished;

	AnimSCRIPT **g_s_state_anim[MAX_PARTS];
	class ActionSCRIPT **g_s_state_action; 
	class ConditionSCRIPT **g_s_state_condition;
	int state_before_special;
	bool *g_s_state_executing;
};


#endif

